"""
Module containing the scaling functions.
"""

import time
import socket
import os

import json

import numpy as np

from binarycpython.utils.grid import Population


def dummy_parsefunction(self, output):
    """
    Dummy parsing function
    """
    pass


def get_mp_results(population, cpu_list, amt_repeats, total_systems):
    """
    Function that runs a population <amt_repeats> for each
    cpu amount (which is picked from cpu_list)
    """

    population.set(evolution_type="mp")
    mp_dict = {}
    for cpu_amt in cpu_list:

        mp_times = []

        population.set(amt_cores=cpu_amt)

        #
        for _ in range(amt_repeats):
            total_mp_start = time.time()
            population.evolve_population()
            total_mp = time.time() - total_mp_start

            print(
                "MP ({} nodes) run with {} systems: {}".format(
                    cpu_amt, total_systems, total_mp
                )
            )
            mp_times.append(total_mp)

        mp_dict[str(cpu_amt)] = mp_times

    return mp_dict


def get_linear_results(population, amt_repeats, total_systems):
    """
    Function that runs the population linearly (using 1 core) <amt_repeats> times
    """

    population.set(evolution_type="linear")
    linear_times = []
    for _ in range(amt_repeats):
        total_lin_start = time.time()
        population.evolve_population()
        total_lin = time.time() - total_lin_start

        print("linear run with {} systems: {}".format(total_systems, total_lin))
        linear_times.append(total_lin)

    return linear_times


def run_systems_for_scaling_comparison(settings_dict):
    """
    Function that runs the systems for the scaling comparison
    """

    amount_of_cpus = settings_dict["amount_of_cpus"]
    amount_of_cores = settings_dict["amount_of_cores"]
    amt_repeats = settings_dict["amt_repeats"]
    stepsize_cpus = settings_dict["stepsize_cpus"]
    testcase = settings_dict["testcase"]
    plot_dir = settings_dict["plot_dir"]
    result_dir = settings_dict["result_dir"]

    resolutions = settings_dict["resolutions"]

    # For each set of resolutions
    for resolution in resolutions:
        # Some calculated values
        total_systems = int(np.prod([el for el in resolution.values()]))
        hostname = socket.gethostname()

        # Generate the range of cpu numbers
        cpu_list = np.arange(1, amount_of_cpus + 1, stepsize_cpus)
        if not cpu_list[-1] == amount_of_cpus:
            cpu_list = np.append(cpu_list, np.array([amount_of_cpus]))

        ##################################################################
        # Create dictionary in which to store all the results:
        result_dict = {}

        #
        result_dict["amt_systems"] = total_systems
        result_dict["hostname"] = hostname
        result_dict["amt_logical_cores"] = amount_of_cpus
        result_dict["amt_of_physical_cores"] = amount_of_cores
        result_dict["testcase"] = testcase

        #################
        # Configuring population
        test_pop = Population()

        test_pop.set(
            verbose=1, binary=1, parse_function=dummy_parsefunction,
        )

        test_pop.add_grid_variable(
            name="lnm1",
            longname="Primary mass",
            valuerange=[1, 100],
            resolution="{}".format(resolution["M_1"]),
            spacingfunc="const(math.log(1), math.log(100), {})".format(
                resolution["M_1"]
            ),
            precode="M_1=math.exp(lnm1)",
            probdist="three_part_powerlaw(M_1, 0.1, 0.5, 1.0, 100, -1.3, -2.3, -2.3)*M_1",
            dphasevol="dlnm1",
            parameter_name="M_1",
            condition="",  # Impose a condition on this grid variable. Mostly for a check for yourself
        )

        test_pop.add_grid_variable(
            name="q",
            longname="Mass ratio",
            valuerange=["0.1/M_1", 1],
            resolution="{}".format(resolution["q"]),
            spacingfunc="const(0.1/M_1, 1, {})".format(resolution["q"]),
            probdist="flatsections(q, [{'min': 0.1/M_1, 'max': 0.8, 'height': 1}, {'min': 0.8, 'max': 1.0, 'height': 1.0}])",
            dphasevol="dq",
            precode="M_2 = q * M_1",
            parameter_name="M_2",
            condition="",  # Impose a condition on this grid variable. Mostly for a check for yourself
        )

        test_pop.add_grid_variable(
            name="logper",
            longname="log(Orbital_Period)",
            valuerange=[-2, 12],
            resolution="{}".format(resolution["per"]),
            spacingfunc="np.linspace(-2, 12, {})".format(resolution["per"]),
            precode="orbital_period = 10** logper\n",  # TODO:
            probdist="gaussian(logper,4.8, 2.3, -2.0, 12.0)",
            parameter_name="orbital_period",
            dphasevol="dln10per",
        )

        #######################################################################################
        # Execute grids

        # Linear runs
        linear_times = get_linear_results(test_pop, amt_repeats, total_systems)
        result_dict["linear"] = linear_times

        #######################################################################################
        # MP runs
        mp_dict = get_mp_results(test_pop, cpu_list, amt_repeats, total_systems)
        result_dict["mp"] = mp_dict

        print(result_dict)

        # Write to file and make sure the directory exists.
        os.makedirs(result_dir, exist_ok=True)
        with open(
            os.path.join(
                result_dir, "{}_{}_systems.json".format(hostname, total_systems)
            ),
            "w",
        ) as f:
            f.write(json.dumps(result_dict, indent=4))
